import unittest

from unittest.mock import Mock, patch

from bitdotio.api_client import ApiClient
from bitdotio._bitdotio import API_VERSION


class TestApiClient(unittest.TestCase):
    token = "v2_testtoken"

    def setUp(self) -> None:
        self.api_client = ApiClient(self.token, API_VERSION)

    def test_init(self) -> None:
        self.assertEqual(self.api_client._access_token, self.token)
        self.assertEqual(self.api_client._api_version, API_VERSION)
        self.assertIsNotNone(self.api_client._session)
        headers = self.api_client._session.headers
        self.assertEqual(headers["Accept"], "application/json")
        self.assertEqual(headers["Authorization"], f"Bearer {self.token}")

    def test_url(self) -> None:
        expected = f"https://api.bit.io/{API_VERSION}/foo/bar"
        result = self.api_client._url("/foo/bar")
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.Session.request")
    def test_request(self, mock_request: Mock) -> None:
        url = "/foo"
        expected_url = f"https://api.bit.io/{API_VERSION}/foo"
        test_cases = [
            ("GET", None),
            ("POST", {"foo": "bar"}),
            ("PATCH", {"foo": "bar"}),
            ("DELETE", None),
        ]
        for method, body in test_cases:
            self.api_client.request(method, url, json=body)
            mock_request.assert_called_with(method, expected_url, json=body)

    @patch("bitdotio.api_client.Session.request")
    def test_get(self, mock_request: Mock) -> None:
        url = "/foo"
        expected_method = "GET"
        expected_url = f"https://api.bit.io/{API_VERSION}/foo"
        self.api_client.get(url)
        mock_request.assert_called_once_with(expected_method, expected_url)

    @patch("bitdotio.api_client.Session.request")
    def test_post(self, mock_request: Mock) -> None:
        url = "/foo"
        expected_method = "POST"
        expected_url = f"https://api.bit.io/{API_VERSION}/foo"
        body = {"foo": "bar"}
        self.api_client.post(url, json=body)
        mock_request.assert_called_once_with(expected_method, expected_url, json=body)

    @patch("bitdotio.api_client.Session.request")
    def test_patch(self, mock_request: Mock) -> None:
        url = "/foo"
        expected_method = "PATCH"
        expected_url = f"https://api.bit.io/{API_VERSION}/foo"
        body = {"foo": "bar"}
        self.api_client.patch(url, json=body)
        mock_request.assert_called_once_with(expected_method, expected_url, json=body)

    @patch("bitdotio.api_client.Session.request")
    def test_delete(self, mock_request: Mock) -> None:
        url = "/foo"
        expected_method = "DELETE"
        expected_url = f"https://api.bit.io/{API_VERSION}/foo"
        self.api_client.delete(url)
        mock_request.assert_called_once_with(expected_method, expected_url)



import typing as t
from urllib.parse import urljoin

from requests import Response, Session

from bitdotio._version import __version__


class ApiClient:

    __slots__ = ("_access_token", "_api_version", "_session")

    def __init__(self, access_token: str, api_version: str) -> None:
        self._access_token = access_token
        self._api_version = api_version

        self._session = Session()
        self._session.headers.update(
            {
                "Accept": "application/json",
                "Authorization": f"Bearer {self._access_token}",
                "User-Agent": f"python-bitdotio-sdk/{__version__}"
            }
        )

    def _url(self, path: str) -> str:
        return urljoin(f"https://api.bit.io", f"{self._api_version}{path}")

    def request(self, method: str, url: str, **kwargs: t.Any) -> Response:
        return self._session.request(method, self._url(url), **kwargs)

    def get(self, url: str, **kwargs: t.Any) -> Response:
        return self.request("GET", url, **kwargs)

    def post(self, url: str, json: t.Any = None, **kwargs: t.Any) -> Response:
        return self.request("POST", url, json=json, **kwargs)

    def patch(self, url: str, json: t.Any = None, **kwargs: t.Any) -> Response:
        return self.request("PATCH", url, json=json, **kwargs)

    def delete(self, url: str, **kwargs: t.Any) -> Response:
        return self.request("DELETE", url, **kwargs)

    def set_header(self, header: str, value: str) -> None:
        self._session.headers.update({header: value})



import unittest

from bitdotio.utils import validate_database_name, validate_token


class TestUtils(unittest.TestCase):
    def test_validate_token_ok(self) -> None:
        validate_token("v2_testtoken")

    def test_validate_token_error(self) -> None:
        test_cases = ["v2_", "v1_testtoken", "v2__.--_--", "v2testtoken", "testtoken"]
        for test_case in test_cases:
            with self.assertRaises(ValueError):
                validate_token(test_case)

    def test_validate_database_name_ok(self) -> None:
        validate_database_name("my/db")
        validate_database_name("m/d")
        validate_database_name("my/db/with/some/slashes")
        validate_database_name("my/db/with.some+other$things")

    def test_validate_database_name_error(self) -> None:
        test_cases = [
            "/mydb",
            "mydb",
            "myExtremelyLongUserNameWhichIsDefinitelyWayTooLong/db",
            "me/myExtremelyLongDatabaseNameWhichIsDefinitelyWayTooLong",
            "-/db",
            "./db",
            "me/",
        ]
        for test_case in test_cases:
            with self.assertRaises(ValueError):
                validate_database_name(test_case)



import io
import unittest
import uuid
from unittest.mock import Mock, patch, ANY

from requests_toolbelt import MultipartEncoder

from bitdotio import bitdotio
from bitdotio._bitdotio import ApiError


class ApiTestCase(unittest.TestCase):
    token = "v2_testtoken"

    def setUp(self) -> None:
        self.b = bitdotio(self.token)


class TestApiMethods(ApiTestCase):
    @patch("bitdotio.api_client.ApiClient.request")
    def test_query_ok(self, mock_request: Mock) -> None:
        db_name = "my/db"
        query_str = "SELECT 1"
        data_format = "objects"
        expected = {"foo": "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected

        result = self.b.query("my/db", "SELECT 1")

        # Test without data_format
        self.assertEqual(result, expected)
        mock_request.assert_called_once_with(
            "POST",
            "/query",
            json={"database_name": db_name, "query_string": query_str},
        )

        mock_request.reset_mock()

        # Test with data_format
        result = self.b.query("my/db", "SELECT 1", data_format=data_format)
        self.assertEqual(result, expected)
        mock_request.assert_called_once_with(
            "POST",
            f"/query?data_format={data_format}",
            json={"database_name": db_name, "query_string": query_str},
        )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_query_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.query("my/db", "SELECT 1")
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_list_databases_ok(self, mock_request: Mock) -> None:
        expected = ["foo", "bar"]
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = {"databases": expected}
        result = self.b.list_databases()
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_list_databases_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.list_databases()
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_database_ok(self, mock_request: Mock) -> None:
        expected = {"foo", "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 201
        mock_request.return_value.json.return_value = expected
        result = self.b.create_database("my-db")
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_database_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.create_database("my-db")
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_database_ok(self, mock_request: Mock) -> None:
        expected = {"foo", "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 201
        mock_request.return_value.json.return_value = expected
        result = self.b.get_database("my/db")
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_database_invalid_db_name(self, _: Mock) -> None:
        with self.assertRaises(ValueError):
            self.b.get_database("mydb")

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_database_error(self, mock_request: Mock) -> None:
        status_code = 401
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.get_database("my/db")
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_update_database_ok(self, mock_request: Mock) -> None:
        expected = {"foo", "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected
        result = self.b.update_database("my/db", is_private=False)
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_update_database_invalid_db_name(self, _: Mock) -> None:
        with self.assertRaises(ValueError):
            self.b.update_database("mydb", is_private=False)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_update_database_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.update_database("my/db", is_private=False)
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_delete_database_ok(self, mock_request: Mock) -> None:
        expected = {"foo", "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected
        result = self.b.delete_database("my/db")
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_delete_database_invalid_db_name(self, _: Mock) -> None:
        with self.assertRaises(ValueError):
            self.b.delete_database("mydb")

    @patch("bitdotio.api_client.ApiClient.request")
    def test_delete_database_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.delete_database("my/db")
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_list_service_accounts_ok(self, mock_request: Mock) -> None:
        expected = ["foo", "bar"]
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = {"service_accounts": expected}
        result = self.b.list_service_accounts()
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_list_service_accounts_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.list_service_accounts()
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_service_acount_ok(self, mock_request: Mock) -> None:
        expected = {"foo": "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected
        result = self.b.get_service_account(str(uuid.uuid4()))
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_service_account_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.get_service_account(str(uuid.uuid4()))
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_service_account_key_ok(self, mock_request: Mock) -> None:
        expected = {"foo": "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected
        result = self.b.create_service_account_key(str(uuid.uuid4()))
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_service_account_key_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.create_service_account_key(str(uuid.uuid4()))
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_revoke_service_account_keys_ok(self, mock_request: Mock) -> None:
        expected = None
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected
        result = self.b.revoke_service_account_keys(str(uuid.uuid4()))
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_revoke_service_account_keys_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.revoke_service_account_keys(str(uuid.uuid4()))
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_key_ok(self, mock_request: Mock) -> None:
        expected = {"foo": "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected
        result = self.b.create_key()
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_key_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.create_key()
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_revoke_keys_ok_all(self, mock_request: Mock) -> None:
        expected = None
        expected_url = "/api-key/"
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected
        result = self.b.revoke_keys()
        self.assertEqual(result, expected)
        mock_request.assert_called_once_with("DELETE", expected_url)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_revoke_keys_ok_single(self, mock_request: Mock) -> None:
        expected = None
        expected_url = f"/api-key/?api_key={self.token}"
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 200
        mock_request.return_value.json.return_value = expected
        result = self.b.revoke_keys(self.token)
        self.assertEqual(result, expected)
        mock_request.assert_called_once_with("DELETE", expected_url)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_revoked_keys_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.revoke_keys()
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)


class TestImports(ApiTestCase):
    @patch("requests_toolbelt.multipart.encoder.uuid4")
    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_import_job_file_ok(
        self, mock_request: Mock, mock_uuid4: Mock
    ) -> None:
        mock_uuid4.return_value = uuid.uuid4()

        file = io.BytesIO(b"foo")
        mock_request.return_value.ok = True
        mock_request.return_value.json.return_value = {"foo": "bar"}
        self.b.create_import_job("my/db", "table", infer_header="first_row", file=file)
        data = {"table_name": "table", "infer_header": "first_row"}
        data = MultipartEncoder(
            fields={
                "file": (data["table_name"], file),
                **data,
            }
        )
        mock_request.assert_called_once_with(
            "POST",
            "/db/my/db/import/",
            json=None,
            data=ANY,
            headers={"Content-Type": data.content_type},
        )
        self.assertEqual(
            data.fields,
            mock_request.call_args.kwargs["data"].fields,
        )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_import_job_url_ok(self, mock_request: Mock) -> None:
        url = "https://example.org/file.csv"
        mock_request.return_value.ok = True
        mock_request.return_value.json.return_value = {"foo": "bar"}
        self.b.create_import_job("my/db", "table", schema_name="schema", file_url=url)
        mock_request.assert_called_once_with(
            "POST",
            "/db/my/db/import/",
            json=None,
            data={"table_name": "table", "schema_name": "schema", "file_url": url},
            headers={},
        )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_import_job_invalid_db_name(self, _: Mock) -> None:
        with self.assertRaisesRegex(ValueError, "Invalid database name"):
            self.b.create_import_job(
                "mydb",
                "mytable",
                file_url="https://example.org/myfile.csv",
            )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_import_job_missing_file_and_url(self, _: Mock) -> None:
        with self.assertRaisesRegex(ValueError, "Must provide file XOR file_url"):
            self.b.create_import_job(
                "my/db",
                "mytable",
            )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_import_job_both_file_and_url(self, _: Mock) -> None:
        with self.assertRaisesRegex(ValueError, "Must provide file XOR file_url"):
            self.b.create_import_job(
                "my/db",
                "mytable",
                file=io.BytesIO(b"foo"),
                file_url="https://example.org/myfile.csv",
            )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_import_job_error(self, mock_request: Mock) -> None:
        url = "https://example.org/file.csv"
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.create_import_job("my/db", "table", file_url=url)
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_import_job_ok(self, mock_request: Mock) -> None:
        expected = {"foo", "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 201
        mock_request.return_value.json.return_value = expected
        result = self.b.get_import_job(str(uuid.uuid4()))
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_import_job_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.get_import_job(str(uuid.uuid4()))
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)


class TestExports(ApiTestCase):
    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_export_job_query_ok(self, mock_request: Mock) -> None:
        mock_request.return_value.ok = True
        mock_request.return_value.json.return_value = {"foo": "bar"}
        self.b.create_export_job("my/db", query_string="select 1", file_name="foo.csv")
        mock_request.assert_called_once_with(
            "POST",
            "/db/my/db/export/",
            json={
                "query_string": "select 1",
                "file_name": "foo.csv",
                "export_format": "csv",
            },
        )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_export_job_table_ok(self, mock_request: Mock) -> None:
        mock_request.return_value.ok = True
        mock_request.return_value.json.return_value = {"foo": "bar"}
        self.b.create_export_job(
            "my/db",
            table_name="table",
            schema_name="schema",
            export_format="parquet",
        )
        mock_request.assert_called_once_with(
            "POST",
            "/db/my/db/export/",
            json={
                "table_name": "table",
                "schema_name": "schema",
                "export_format": "parquet",
            },
        )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_export_job_table_no_schema_ok(self, mock_request: Mock) -> None:
        mock_request.return_value.ok = True
        mock_request.return_value.json.return_value = {"foo": "bar"}
        self.b.create_export_job(
            "my/db",
            table_name="table",
            export_format="xls",
        )
        mock_request.assert_called_once_with(
            "POST",
            "/db/my/db/export/",
            json={
                "table_name": "table",
                "schema_name": "public",
                "export_format": "xls",
            },
        )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_export_job_invalid_db_name(self, _: Mock) -> None:
        with self.assertRaisesRegex(ValueError, "Invalid database name"):
            self.b.create_export_job("mydb", query_string="select 1")

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_export_job_missing_query_and_table(self, _: Mock) -> None:
        with self.assertRaisesRegex(
            ValueError, "Must provide query_string XOR table_name"
        ):
            self.b.create_export_job("my/db")

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_export_job_both_query_and_table(self, _: Mock) -> None:
        with self.assertRaisesRegex(
            ValueError, "Must provide query_string XOR table_name"
        ):
            self.b.create_export_job(
                "my/db",
                query_string="select 1",
                table_name="table",
                schema_name="schema",
            )

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_export_job_missing_table(self, _: Mock) -> None:
        with self.assertRaisesRegex(
            ValueError, "Must provide query_string XOR table_name"
        ):
            self.b.create_export_job("my/db", schema_name="schema")

    @patch("bitdotio.api_client.ApiClient.request")
    def test_create_export_job_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.create_export_job("my/db", query_string="q", export_format="xls")
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_export_job_ok(self, mock_request: Mock) -> None:
        expected = {"foo", "bar"}
        mock_request.return_value.ok = True
        mock_request.return_value.status_code = 201
        mock_request.return_value.json.return_value = expected
        result = self.b.get_export_job(str(uuid.uuid4()))
        self.assertEqual(result, expected)

    @patch("bitdotio.api_client.ApiClient.request")
    def test_get_export_job_error(self, mock_request: Mock) -> None:
        status_code = 400
        error_data = {"error": "whoops"}
        mock_request.return_value.ok = False
        mock_request.return_value.status_code = status_code
        mock_request.return_value.json.return_value = error_data
        with self.assertRaises(ApiError) as cm:
            self.b.get_export_job(str(uuid.uuid4()))
            self.assertEqual(cm.exception.status_code, status_code)
            self.assertEqual(cm.exception.data, error_data)



import unittest
from unittest.mock import Mock, patch

from bitdotio import bitdotio


class TestConnect(unittest.TestCase):
    token = "v2_testtoken"

    def setUp(self) -> None:
        self.b = bitdotio(self.token)

    def test_min_max_conn_validation(
        self,
    ) -> None:
        test_cases = [(-1, 0), (0, -1), (0, 0), (1, 0)]
        for min_conn, max_conn in test_cases:
            with self.assertRaises(ValueError):
                bitdotio(self.token, min_conn=min_conn, max_conn=max_conn)

    @patch("bitdotio._bitdotio._BitV2._create_pool")
    def test_connect(self, mock_create_pool: Mock) -> None:
        db_1, db_2 = "my/db1", "my/db2"
        with self.b.pooled_connection(db_1):
            pass

        # get/put should be called and there should be one pool after connecting once
        self.assertEqual(len(self.b._pools), 1)
        mock_create_pool.return_value.getconn.assert_called_once()
        mock_create_pool.return_value.putconn.assert_called_once()
        # validate that connections are exposed inside an internal context manager
        mock_create_pool.return_value.getconn.return_value.__exit__.assert_called_once()

        mock_create_pool.return_value.getconn.reset_mock()
        mock_create_pool.return_value.putconn.reset_mock()
        with self.b.pooled_connection(db_1):
            pass

        # get/put should be called and there should be one pool after connecting to the
        # same database the second time.
        self.assertEqual(len(self.b._pools), 1)
        mock_create_pool.return_value.getconn.assert_called_once()
        mock_create_pool.return_value.putconn.assert_called_once()
        mock_create_pool.return_value.getconn.return_value.__exit__.assert_called_once()

        mock_create_pool.return_value.getconn.reset_mock()
        mock_create_pool.return_value.putconn.reset_mock()
        with self.b.pooled_connection(db_2):
            pass

        # get/put should be called and there should be two pools after connecting to
        # the second database.
        self.assertEqual(len(self.b._pools), 2)
        mock_create_pool.return_value.getconn.assert_called_once()
        mock_create_pool.return_value.putconn.assert_called_once()
        mock_create_pool.return_value.getconn.return_value.__exit__.assert_called_once()

    @patch("bitdotio._bitdotio._BitV2._create_pool")
    def test_cursor(self, mock_create_pool: Mock) -> None:
        db_1, db_2 = "my/db1", "my/db2"
        with self.b.pooled_cursor(db_1):
            pass

        # get/put should be called and there should be one pool after connecting once
        self.assertEqual(len(self.b._pools), 1)
        mock_create_pool.return_value.getconn.assert_called_once()
        mock_create_pool.return_value.putconn.assert_called_once()
        # validate that connections are exposed inside an internal context manager
        mock_create_pool.return_value.getconn.return_value.__exit__.assert_called_once()

        mock_create_pool.return_value.getconn.reset_mock()
        mock_create_pool.return_value.putconn.reset_mock()
        with self.b.pooled_cursor(db_1):
            pass

        # get/put should be called and there should be one pool after connecting to the
        # same database the second time.
        self.assertEqual(len(self.b._pools), 1)
        mock_create_pool.return_value.getconn.assert_called_once()
        mock_create_pool.return_value.putconn.assert_called_once()
        mock_create_pool.return_value.getconn.return_value.__exit__.assert_called_once()

        mock_create_pool.return_value.getconn.reset_mock()
        mock_create_pool.return_value.putconn.reset_mock()
        with self.b.pooled_cursor(db_2):
            pass

        # get/put should be called and there should be two pools after connecting to
        # the second database.
        self.assertEqual(len(self.b._pools), 2)
        mock_create_pool.return_value.getconn.assert_called_once()
        mock_create_pool.return_value.putconn.assert_called_once()
        mock_create_pool.return_value.getconn.return_value.__exit__.assert_called_once()



import os
import unittest
import tempfile
from unittest.mock import Mock, patch

from bit.bit import bitio
from click.testing import CliRunner

"""
click.testing.CliRunner is not thread safe! This will likely cause
future tests affecting interpreter state to fail.

`bitio.params[0].required = True` in teardown addresses the case where
a test of '--help' functionality without a key changes the bitio 'key'
parameter requirement to "False," causing later tests of the key
requirement to fail.
"""


@patch.dict(os.environ, {"BITIO_KEY": ""})
class TestCLI(unittest.TestCase):
    # Test token that passes regex validation
    token = "v2_testtoken"

    def setUp(self) -> None:
        self.runner = CliRunner()

    def tearDown(self) -> None:
        bitio.params[0].required = True

    def cmd(self, *args):
        return ["-k", self.token, *args]

    def test_help(self):
        result = self.runner.invoke(bitio, ["--help", "bitio"])
        self.assertEqual(result.exit_code, 0)

    def test_no_args(self):
        """Test that bit with no arguments returns same as with --help"""
        res1 = self.runner.invoke(bitio)
        res2 = self.runner.invoke(bitio, ["--help", "bitio"])
        self.assertEqual(res1.output, res2.output)

    def test_help_no_key(self):
        """Test that help for query, import, repo commands are accessible without key"""
        query_result = self.runner.invoke(bitio, ["query", "--help", "bitio"])
        db_result = self.runner.invoke(bitio, ["db", "--help", "bitio"])

        self.assertEqual(query_result.exit_code, 0)
        self.assertEqual(db_result.exit_code, 0)

    @patch("bitdotio._bitdotio._BitV2.get_database", return_value={})
    def test_bitio_key_env_var(self, mock_get_database: Mock):
        # Check that we can omit -k/--key and use BITIO_KEY env var instead. Use db
        # info command for this purpose since it's the simplest.
        os.environ["BITIO_KEY"] = self.token
        cmd = ["db", "info", "-d", "my/db"]
        result = self.runner.invoke(bitio, cmd)
        self.assertEqual(result.exit_code, 0)
        mock_get_database.assert_called_once()
        del os.environ["BITIO_KEY"]

    @patch("bitdotio._bitdotio._BitV2.query", return_value={})
    def test_query_ok_query_str(self, mock_query: Mock):
        cmd = self.cmd("query", "-d", "my/db", "-q", "select 1")
        result = self.runner.invoke(bitio, cmd)
        self.assertEqual(result.exit_code, 0)
        mock_query.assert_called_once_with("my/db", "select 1")

    @patch("bitdotio._bitdotio._BitV2.query", return_value={})
    def test_query_ok_query_file(self, mock_query: Mock):
        query = "select 1"
        with tempfile.TemporaryDirectory() as temp_path:
            file_path = os.path.join(temp_path, "query.sql")
            with open(file_path, "w+") as f:
                f.write(query)

            cmd = self.cmd("query", "-d", "my/db", "-qf", file_path, "-o")
            result = self.runner.invoke(bitio, cmd)
            self.assertEqual(result.exit_code, 0)
            mock_query.assert_called_once_with(
                "my/db", "select 1", data_format="objects"
            )

    def test_query_error(self):
        test_cases = [("-d", "my/db"), ("-q", "select 1", "-o"), ("-qf", "file.sql")]
        for test_case in test_cases:
            cmd = self.cmd("query", *test_case)
            result = self.runner.invoke(bitio, cmd)
            self.assertNotEqual(result.exit_code, 0)

    @patch("bitdotio._bitdotio._BitV2.list_databases", return_value=[])
    def test_db_list_ok(self, mock_list_databases: Mock):
        cmd = self.cmd("db", "list")
        result = self.runner.invoke(bitio, cmd)
        self.assertEqual(result.exit_code, 0)
        mock_list_databases.assert_called_once()

    @patch("bitdotio._bitdotio._BitV2.create_database", return_value=[])
    def test_db_create_ok(self, mock_create_database: Mock):
        cmd = self.cmd("db", "create", "-n", "my/db", "-p", "false")
        result = self.runner.invoke(bitio, cmd)
        self.assertEqual(result.exit_code, 0)
        mock_create_database.assert_called_once_with("my/db", is_private=False)

    def test_db_create_error(self):
        cmd = self.cmd("db", "create", "-p", "false")
        result = self.runner.invoke(bitio, cmd)
        self.assertNotEqual(result.exit_code, 0)

    @patch("bitdotio._bitdotio._BitV2.get_database", return_value=[])
    def test_db_info_ok(self, mock_get_database: Mock):
        cmd = self.cmd("db", "info", "-d", "my/db")
        result = self.runner.invoke(bitio, cmd)
        self.assertEqual(result.exit_code, 0)
        mock_get_database.assert_called_once_with("my/db")

    def test_db_info_error(self):
        cmd = self.cmd("db", "info")
        result = self.runner.invoke(bitio, cmd)
        self.assertNotEqual(result.exit_code, 0)

    @patch("bitdotio._bitdotio._BitV2.update_database", return_value=[])
    def test_db_update_ok(self, mock_update_database: Mock):
        test_cases = [
            [],
            ("-p", "false"),
            ("-n", "your/db"),
            ("-n", "your/db", "-p", "false"),
        ]
        for test_case in test_cases:
            cmd = self.cmd("db", "update", "-d", "my/db", *test_case)
            result = self.runner.invoke(bitio, cmd)
            self.assertEqual(result.exit_code, 0)
            mock_update_database.assert_called_once()
            mock_update_database.reset_mock()

    def test_db_update_error(self):
        cmd = self.cmd("db", "update", "-p", "false")
        result = self.runner.invoke(bitio, cmd)
        self.assertNotEqual(result.exit_code, 0)

    @patch("bitdotio._bitdotio._BitV2.delete_database", return_value=[])
    def test_db_delete_ok(self, mock_delete_database: Mock):
        cmd = self.cmd("db", "delete", "-d", "my/db")
        result = self.runner.invoke(bitio, cmd)
        self.assertEqual(result.exit_code, 0)
        mock_delete_database.assert_called_once_with("my/db")

    def test_db_delete_error(self):
        cmd = self.cmd("db", "delete")
        result = self.runner.invoke(bitio, cmd)
        self.assertNotEqual(result.exit_code, 0)



#!/usr/bin/env python3
import os
import click
import sys
import json

from bitdotio import __version__, bitdotio


USER_AGENT = f"python-bitdotio-cli/{__version__}"


def printj(obj):
    print(json.dumps(obj))


# Allows overriding the required `-k` when subcommands have `--help`
# From https://stackoverflow.com/questions/55818737/python-click-application-required-parameters-have-precedence-over-sub-command-he
class IgnoreRequiredWithHelp(click.Group):
    def parse_args(self, ctx, args):
        try:
            return super().parse_args(ctx, args)
        except click.MissingParameter:
            if "--help" not in args:
                raise

            # HACK: if just doing --help, pass in a phony key that passes regex
            # validation.
            return super().parse_args(ctx, ["-k", "v2_help", "--help"])


@click.version_option(__version__, '-v', '--version', is_flag=True, help="Print bit version and exit")
@click.group(cls=IgnoreRequiredWithHelp)
@click.option(
    "-k",
    "--key",
    required=False,
    help="Your bit.io API key, available when you click the connect button in bit.io",
)
@click.pass_context
def bitio(ctx, key):
    if key is None:
        key = os.environ.get("BITIO_KEY")
        if key is None:
            raise click.MissingParameter(
                "Your bit.io key must be set either using the -k/--key argument or as the environment variable BITIO_KEY"
            )

    b = bitdotio(key)
    b._api_client.set_header("User-Agent", USER_AGENT)
    ctx.obj = b


# bit -k <api-key> query -d <db-name> -q "SELECT 1"
@bitio.command()
@click.option("-d", "--database", required=True, help="Name of the database to query")
@click.option("-q", "--query", required=False, help="The SQL to run.")
@click.option(
    "-qf",
    "--query_file",
    required=False,
    help="A SQL file to run, containing a single query. Use - for stdin.",
    type=click.File("r"),
)
@click.option(
    "-o",
    "--objects",
    is_flag=True,
    help="Return data as an array of JSON objects instead of row arrays",
)
@click.pass_obj
def query(b, database, query, query_file, objects):
    if query:
        query_string = query
    elif query_file:
        with query_file:
            query_string = query_file.read()
    else:
        raise click.UsageError("One of --query or --query_file is required")

    if objects:
        printj(b.query(database, query_string, data_format="objects"))
    else:
        printj(b.query(database, query_string))


@bitio.group()
@click.pass_obj
def db(b):
    pass


@db.command()
@click.pass_obj
def list(b):
    printj(b.list_databases())


@db.command()
@click.option("-n", "--name", required=True, help="The database name")
@click.option(
    "-p",
    "--is_private",
    show_default=True,
    default=True,
    help="Whether or not the database is set to private",
)
@click.pass_obj
def create(b, name, is_private):
    printj(b.create_database(name, is_private=is_private))


@db.command()
@click.option("-d", "--database", required=True, help="Name of the database to show")
@click.pass_obj
def info(b, database):
    printj(b.get_database(database))


@db.command()
@click.option("-d", "--database", required=True, help="Name of the database to show")
@click.option(
    "-n", "--name", required=False, default=None, help="New name for the database"
)
@click.option(
    "-p",
    "--is_private",
    required=False,
    default=None,
    help="Whether or not the database is set to private",
)
@click.pass_obj
def update(b, database, name, is_private):
    printj(b.update_database(database, name=name, is_private=is_private))


@db.command()
@click.option("-d", "--database", required=True, help="Name of the database to show")
@click.pass_obj
def delete(b, database):
    b.delete_database(database)
    print(f"Successfully deleted database: {database}")


def main():
    try:
        bitio(auto_envvar_prefix="BITIO")
    except click.UsageError as e:
        raise e
    except Exception as e:
        print(e)
        sys.exit(3)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import functools
import sys
import typing as t
from contextlib import contextmanager

from requests import Response
from requests_toolbelt import MultipartEncoder

from bitdotio.api_client import ApiClient
from bitdotio.utils import (
    prune_body,
    validate_database_name,
    validate_min_max_conn,
    validate_token,
)

API_VERSION = "v2beta"


def bitdotio(
    access_token: str,
    api_version: str = API_VERSION,
    min_conn: int = 0,
    max_conn: int = 100,
):
    return _BitV2(access_token, api_version, min_conn, max_conn)


class ApiError(Exception):
    def __init__(self, status_code: int, data: t.Any) -> None:
        self.status_code = status_code
        self.data = data

    def __str__(self) -> str:
        return f"ApiError status_code={self.status_code} data={self.data}"


def api_method(
    returning: t.Optional[str] = None,
) -> t.Callable[[t.Callable[..., Response]], t.Callable[..., t.Any]]:
    def decorator(method: t.Callable[..., Response]) -> t.Callable[..., t.Any]:
        @functools.wraps(method)
        def wrapper(*args, **kwargs):
            response = method(*args, **kwargs)
            body = response.json()
            if response.ok:
                if returning:
                    return body[returning]
                return body

            raise ApiError(response.status_code, body)

        return wrapper

    return decorator


class _BitV2:

    _port = 5432
    _host = "db.bit.io"

    def __init__(
        self,
        access_token: str,
        api_version: str,
        min_conn: int,
        max_conn: int,
    ) -> None:
        validate_token(access_token)
        validate_min_max_conn(min_conn, max_conn)

        self._access_token = access_token
        self._api_client = ApiClient(access_token, api_version)
        self._min_conn = min_conn
        self._max_conn = max_conn

        self._pools = {}

    def __repr__(self):
        return "<bitdotio SDK object: v2>"

    def _get_conn_string(self, db_name):
        db = db_name
        user = "api_user"
        password = self._access_token
        host = self._host
        port = self._port
        return f"dbname={db} user={user} password={password} host={host} port={port} sslmode=require"

    @staticmethod
    def _create_pool(pool_cls, *args, **kwargs):
        """This is super weird code factorization but it's necessary in order to mock
        out the connection pool in testing. We can't mock ThreadedConnectionPool
        directly since it's not imported at the module level.
        """
        return pool_cls(*args, **kwargs)

    def _get_pool(self, db_name: str):
        pool = self._pools.get(db_name)
        if pool is not None:
            return pool

        try:
            # Threadsafe by default
            from psycopg2.pool import ThreadedConnectionPool
        except ImportError as exc:
            _print_psycopg2_message()
            raise exc

        conn_string = self._get_conn_string(db_name)
        pool = self._create_pool(
            ThreadedConnectionPool,
            self._min_conn,
            self._max_conn,
            conn_string,
        )
        self._pools[db_name] = pool
        return pool

    def get_connection(self, db_name):
        try:
            import psycopg2
        except ImportError as e:
            _print_psycopg2_message()
            raise e

        conn_str = self._get_conn_string(db_name)
        conn = psycopg2.connect(conn_str)

        return conn

    @contextmanager
    def pooled_connection(self, db_name: str):
        pool, conn = None, None
        try:
            pool = self._get_pool(db_name)
            with pool.getconn() as conn:
                yield conn
        finally:
            if pool is not None and conn is not None:
                pool.putconn(conn)

    @contextmanager
    def pooled_cursor(self, db_name: str):
        with self.pooled_connection(db_name) as conn:
            with conn.cursor() as cursor:
                yield cursor

    @api_method()
    def query(self, db_name: str, query_str: str, data_format: t.Optional[str] = None):
        url = f"/query"
        if data_format is not None:
            url += f"?data_format={data_format}"

        request_body = {"database_name": db_name, "query_string": query_str}

        return self._api_client.post(url, json=request_body)

    @api_method(returning="databases")
    def list_databases(self):
        return self._api_client.get("/db/")

    @api_method()
    def create_database(
        self,
        name: str,
        is_private: bool = True,
        storage_limit_bytes: t.Optional[int] = None,
    ):
        request_body = {"name": name, "is_private": is_private}
        if storage_limit_bytes:
            request_body["storage_limit_bytes"] = storage_limit_bytes

        return self._api_client.post("/db/", json=request_body)

    @api_method()
    def get_database(self, db_name: str):
        validate_database_name(db_name)
        return self._api_client.get(f"/db/{db_name}")

    @api_method()
    def update_database(
        self,
        db_name: str,
        name: t.Optional[str] = None,
        is_private: t.Optional[bool] = None,
        storage_limit_bytes: t.Optional[int] = None,
    ):
        validate_database_name(db_name)
        request_body = prune_body(
            {
                "name": name,
                "is_private": is_private,
                "storage_limit_bytes": storage_limit_bytes,
            }
        )

        return self._api_client.patch(f"/db/{db_name}", json=request_body)

    @api_method()
    def delete_database(self, db_name: str):
        validate_database_name(db_name)
        return self._api_client.delete(f"/db/{db_name}")

    @api_method()
    def create_import_job(
        self,
        db_name: str,
        table_name: str,
        schema_name: t.Optional[str] = None,
        infer_header: t.Optional[t.Literal["auto", "first_row", "no_header"]] = None,
        file: t.Optional[t.IO] = None,
        file_url: t.Optional[str] = None,
    ):
        validate_database_name(db_name)
        if bool(file) == bool(file_url):
            raise ValueError("Must provide file XOR file_url")

        data = prune_body(
            {
                "table_name": table_name,
                "schema_name": schema_name,
                "infer_header": infer_header,
                "file_url": file_url,
            }
        )

        # Python<3.10 is limited to 2 GiB multi-part encoded files when using
        # SSL. requests_toolbelt.MultipartEncoder is a workaround for that
        # limitation. We can remove this workaround later if we drop Python<=3.9.
        # Refs: https://github.com/psf/requests/issues/2717
        #       https://bugs.python.org/issue42853
        headers = {}
        if file is not None:
            data = MultipartEncoder(
                fields={
                    **data,
                    'file': (table_name, file),
                }
            )
            headers["Content-Type"] = data.content_type

        return self._api_client.post(
            f"/db/{db_name}/import/",
            data=data,
            headers=headers,
        )
        
    @api_method()
    def get_import_job(self, import_id: str):
        return self._api_client.get(f"/import/{import_id}")

    @api_method()
    def create_export_job(
        self,
        db_name: str,
        query_string: t.Optional[str] = None,
        table_name: t.Optional[str] = None,
        schema_name: t.Optional[str] = None,
        file_name: t.Optional[str] = None,
        export_format: t.Literal["csv", "json", "xls", "parquet"] = "csv",
    ):
        validate_database_name(db_name)
        if bool(query_string) == bool(table_name):
            raise ValueError("Must provide query_string XOR table_name")

        # Explicit schema name is required by the API, but we can default to public
        # here if table_name is given.
        if table_name and not schema_name:
            schema_name = "public"

        request_body = prune_body(
            {
                "query_string": query_string,
                "table_name": table_name,
                "schema_name": schema_name,
                "file_name": file_name,
                "export_format": export_format,
            }
        )

        return self._api_client.post(f"/db/{db_name}/export/", json=request_body)

    @api_method()
    def get_export_job(self, export_id: str):
        return self._api_client.get(f"/export/{export_id}")

    @api_method(returning="service_accounts")
    def list_service_accounts(self):
        return self._api_client.get("/service-account/")

    @api_method()
    def get_service_account(self, service_account_id: str):
        return self._api_client.get(f"/service-account/{service_account_id}")

    @api_method()
    def create_service_account_key(self, service_account_id: str):
        return self._api_client.post(f"/service-account/{service_account_id}/api-key/")

    @api_method()
    def revoke_service_account_keys(self, service_account_id: str):
        return self._api_client.delete(
            f"/service-account/{service_account_id}/api-key/"
        )

    @api_method()
    def create_key(self):
        return self._api_client.post("/api-key/")

    @api_method()
    def revoke_keys(self, api_key: t.Optional[str] = None):
        path = "/api-key/"
        if api_key:
            path += f"?api_key={api_key}"
        return self._api_client.delete(path)

def _print_psycopg2_message():
    print(
        """
It looks like we couldn't import the psycopg2 library!

In order to support different environments, we have a few ways to install the bitdotio package
with or without the psycopg2 dependency.

1. If you already have psycopg2 install, you can install the default bitdotio package:

  pip install bitdotio

2. If you already have Postgres installed, you can install with the psycopg2 dependency:

  pip install bitdotio[psycopg2]

3. If you do not have or cannot install Postgres, you can install with the psycopg2-binary dependency:

  pip install bitdotio[psycopg2-binary]

""",
        file=sys.stderr,
    )



import re


_RE_TOKEN = re.compile(r"^v2_\w+$")
_RE_DB_NAME = re.compile(r"^\w.{0,37}?/.{1,23}$")


def validate_token(token: str):
    if _RE_TOKEN.fullmatch(token) is None:
        raise ValueError("Invalid access token")


def validate_database_name(db_name: str):
    if _RE_DB_NAME.fullmatch(db_name) is None:
        raise ValueError("Invalid database name")


def validate_min_max_conn(min_conn: int, max_conn: int):
    if min_conn < 0 or max_conn < 0:
        raise ValueError("min_conn and max_conn must both be greater than or equal to zero")
    if min_conn >= max_conn:
        raise ValueError("min_conn must be strictly less than max_conn")

def prune_body(body: dict) -> dict:
    return {k: v for k, v in body.items() if v is not None}



# coding: utf-8
import os
import re
from setuptools import setup, find_packages  # noqa: H301


NAME = "bitdotio"
VERSIONFILE = os.path.join(NAME, "_version.py")
RE_VERSION = re.compile(r'^__version__ = "(?P<version>.+?)"$')


def get_version():
    with open(VERSIONFILE, "r") as f:
        match = RE_VERSION.match(f.read().strip())
        if match:
            return match.group("version")

        raise ValueError("Unable to parse version file")


# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["requests >= 2.28.1", "click >= 8.0.1"]

EXTRAS = {
    "psycopg2": [
        "psycopg2>=2.8.6",
    ],
    "psycopg2-binary": [
        "psycopg2-binary>=2.8.6",
    ],
}


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name=NAME,
    version=get_version(),
    description="bit.io Python SDK and CLI",
    author="bit.io",
    author_email="python@bit.io",
    url="https://github.com/bitdotioinc/python-bitdotio",
    keywords=["bit.io", "Database", "bit.io Python SDK"],
    install_requires=REQUIRES,
    extras_require=EXTRAS,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    project_urls={
        "Bug Tracker": "https://github.com/bitdotioinc/python-bitdotio/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    entry_points={
        "console_scripts": [
            "bit = bit.bit:main",
        ]
    },
)
